#ifndef CMIRRORDIALOG_H
#define CMIRRORDIALOG_H

#include <iostream>
#include <gtkmm.h>

// This is a dialog box that allows the user to choose a Zenwalk mirror
// from a list and/or edit the list.
// See COPYING for license information

class CMirrorDialog : public Gtk::Dialog
{
public:
	CMirrorDialog();
	void		Clear();
	void		AddMirror(Glib::ustring mirror);
	void		SetCurrentMirror(Glib::ustring mirror);
	Glib::ustring	GetCurrentMirror();
	void		SaveMirrors(std::ofstream& ofile);
protected:
	void		on_button_edit();
	void		on_button_add();
	void		on_button_remove();
	void		on_treeview_doubleclicked(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column);

	Gtk::Table		_Table;
	Gtk::ScrolledWindow	_Scroll;
	Gtk::Frame		_Frame;

	Gtk::TreeView				_TreeView;
	Gtk::TreeModel::ColumnRecord		_ColumnRecord;
	Glib::RefPtr<Gtk::ListStore>		_RefModel;
	Gtk::TreeModelColumn<Glib::ustring>	_ColMirrors;

	Gtk::VBox	_RightBox;
	Gtk::Button	_Edit;
	Gtk::Button	_Add;
	Gtk::Button	_Remove;

	// This dialog is used for Add/Edit operations.
	Gtk::Dialog	_AddDlg;
	Gtk::Entry	_AddEntry;

	// This dialog is used to confirm mirror removal.
	Gtk::Dialog	_RemoveDlg;
	Gtk::Label	_RemoveLbl;
};

CMirrorDialog::CMirrorDialog() :
_Table(2, 1),
_Edit(Gtk::Stock::EDIT),
_Add(Gtk::Stock::ADD),
_Remove(Gtk::Stock::REMOVE),
_RemoveLbl("Are you sure you want to remove the selected mirror?")
{
	set_title("Select/Edit Mirrors");
	set_default_size(570, 350);
	add_button(Gtk::Stock::CANCEL, -4);
	add_button(Gtk::Stock::OK, 1);
	set_default_response(1);
	get_vbox()->add(_Table);

	// Add the containers to the main table
	_Table.attach(_Frame, 0, 1, 0, 1, Gtk::SHRINK | Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK | Gtk::EXPAND | Gtk::FILL, 6, 6);
	_Table.attach(_RightBox, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 6, 6);

	// Set up the frame and scrollbar containers
	_Frame.add(_Scroll);
	_Scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	_Scroll.add(_TreeView);

	// Set up the box on the right
	_RightBox.set_spacing(6);
	_RightBox.pack_start(_Edit);
	_Edit.signal_clicked().connect(sigc::mem_fun(*this, &CMirrorDialog::on_button_edit));
	_RightBox.pack_start(_Add);
	_Add.signal_clicked().connect(sigc::mem_fun(*this, &CMirrorDialog::on_button_add));
	_RightBox.pack_start(_Remove);
	_Remove.signal_clicked().connect(sigc::mem_fun(*this, &CMirrorDialog::on_button_remove));

	// Set up the column in the mirror list
	_ColumnRecord.add(_ColMirrors);
	_RefModel = Gtk::ListStore::create(_ColumnRecord);
	_TreeView.set_model(_RefModel);
	_TreeView.append_column("Mirrors", _ColMirrors);
	_TreeView.signal_row_activated().connect(sigc::mem_fun(*this, &CMirrorDialog::on_treeview_doubleclicked));

	// Create the Add/Edit dialog box.
	_AddDlg.set_title("Add/Edit Mirror");
	_AddDlg.set_default_size(500, 90);
	_AddDlg.set_border_width(6);
	_AddDlg.get_vbox()->set_spacing(6);
	_AddDlg.get_vbox()->pack_start(_AddEntry);
	_AddDlg.add_button(Gtk::Stock::CANCEL, -4);
	_AddDlg.add_button(Gtk::Stock::OK, 1);
	_AddDlg.set_default_response(1);
	_AddDlg.show_all_children();

	// Create the Remove dialog box.
	_RemoveDlg.set_title("Confirm Mirror Removal");
	_RemoveDlg.set_has_separator(false);
	_RemoveDlg.set_default_size(400, 100);
	_RemoveDlg.get_vbox()->pack_start(_RemoveLbl);
	_RemoveDlg.add_button(Gtk::Stock::NO, -4);
	_RemoveDlg.add_button(Gtk::Stock::YES, 1);
	_RemoveDlg.set_default_response(1);
	_RemoveDlg.show_all_children();

	show_all_children();
}

void CMirrorDialog::SetCurrentMirror(Glib::ustring mirror)
{	// Set the URL of the current mirror.
	Gtk::TreeModel::Children children = _RefModel->children();
	for(Gtk::TreeModel::Children::iterator _iter = children.begin(); _iter != children.end(); ++_iter) {
		if ( (*_iter)[_ColMirrors] == mirror ) {
			_TreeView.get_selection()->select(*_iter);
			return;
		}
	}
	if ( !mirror.find("local://") ) {
		(*(_RefModel->append()))[_ColMirrors] = mirror;
		SetCurrentMirror(mirror);
	}
}

Glib::ustring CMirrorDialog::GetCurrentMirror()
{	// Returns the URL of the currently selected mirror.
	Gtk::TreeModel::iterator	_iter = _TreeView.get_selection()->get_selected();
	if ( !_iter ) return "";
	return (*_iter)[_ColMirrors];
}

void CMirrorDialog::Clear()
{
	_RefModel->clear();
}

void CMirrorDialog::AddMirror(Glib::ustring mirror)
{
	if ( mirror.substr(mirror.length() - 1) == "/" )
		mirror.erase(mirror.length() - 1);
	Gtk::TreeModel::Row	_row = *(_RefModel->append());
	_row[_ColMirrors] = mirror;
}

void CMirrorDialog::SaveMirrors(std::ofstream& ofile)
{
	Gtk::TreeModel::Children	children = _RefModel->children();
	for ( Gtk::TreeModel::Children::iterator _iter = children.begin(); _iter != children.end(); ++_iter )
		ofile << "Mirror=" << (*_iter)[_ColMirrors] << std::endl;
	ofile << "CurrentMirror=" << GetCurrentMirror() << std::endl;
}

void CMirrorDialog::on_button_edit()
{	// Edit the currently selected mirror.
	Gtk::TreeModel::iterator	_iter = _TreeView.get_selection()->get_selected();
	if ( !_iter ) return;
	_AddEntry.set_text((*_iter)[_ColMirrors]);
	if ( _AddDlg.run() == 1 && _AddEntry.get_text().length() > 0 )
		(*_iter)[_ColMirrors] = _AddEntry.get_text();
	_AddDlg.hide();
}

void CMirrorDialog::on_button_add()
{	// Add a new mirror to the list.
	_AddEntry.set_text("http://");
	if ( _AddDlg.run() == 1 && _AddEntry.get_text().length() > 0 )
		AddMirror(_AddEntry.get_text());
	_AddDlg.hide();
}

void CMirrorDialog::on_button_remove()
{	// Remove the currently selected mirror.
	Gtk::TreeModel::iterator	_iter = _TreeView.get_selection()->get_selected();
	if ( !_iter ) return;
	if ( _RemoveDlg.run() == 1 ) _RefModel->erase(_iter);
	_RemoveDlg.hide();
}

void CMirrorDialog::on_treeview_doubleclicked(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
{
	response(1);
}

#endif
