#ifndef CREFRESHER_H
#define CREFRESHER_H
#include <gtkmm.h>
#include <iostream>
#include "CDownloader.h"
#include "common_fun.h"
using namespace Common;

// This class parses package lists, or downloads a mirror package
// list, all while displaying progress via a very simple GUI.
// Use whichever constructor you need to for the specific task
// you wish to be performed.
// See COPYING for license information.

class CRefresher : public Gtk::Dialog
{
public:
	CRefresher(packagemap_t* pkgs, Glib::ustring blacklist, Glib::ustring mirror);
	CRefresher(Glib::ustring mirror);
protected:
	void		on_button_cancel();
	void		parseInstalled();
	void		parseMirror();
	bool		downloadMirror();
	bool		on_timer();
	void		parseFile(Glib::ustring filename);
	void		parseLocal();

	Gtk::ProgressBar	_Progress;
	Gtk::Button		_CancelBut;

	bool		_Parser;	// True if class is a parser, false if it's a downloader.
	packagemap_t*	_Packages;
	Glib::ustring	_Blacklist;
	Glib::ustring	_Mirror;
	CDownloader*	_Downloader;
	bool		_DownloadFallback;
};

CRefresher::CRefresher(packagemap_t* pkgs, Glib::ustring blacklist, Glib::ustring mirror) :
_Parser(true),
_Mirror(mirror),
_Blacklist(blacklist),
_Packages(pkgs)
{	// This constructor is for parsing the package lists
	get_vbox()->pack_start(_Progress);
	set_border_width(20);
	show_all_children();

	Glib::signal_timeout().connect(sigc::mem_fun(*this, &CRefresher::on_timer), 200);
	run();
}

CRefresher::CRefresher(Glib::ustring mirror) :
_Parser(false),
_Downloader(NULL),
_DownloadFallback(false),
_CancelBut(Gtk::Stock::CANCEL),
_Mirror(mirror)
{	// This constructor is for downloading the mirror package list.
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &CRefresher::on_timer), 200);
	system(Glib::ustring("rm -f " + TMPFILE("PACKAGES.TXT")).c_str());
	if ( !mirror.find("local://") ) {
		system(Glib::ustring("cp " + mirror.substr(8) + "/PACKAGES.TXT " + TMPFILE("") + " 2> /dev/null").c_str());
		return;	// Exit if local mirror.
	}
	if ( !mirror.length() ) return;

	WRITELOG("Downloading mirror: " + mirror);
	set_title("Downloading");
	get_vbox()->pack_start(_Progress);
	get_action_area()->pack_start(_CancelBut);
	set_border_width(20);
	show_all_children();

	_CancelBut.signal_clicked().connect(sigc::mem_fun(*this, &CRefresher::on_button_cancel));
	_Progress.set_text("Connecting to: " + _Mirror);
	_Downloader = new CDownloader(_Mirror + "/PACKAGES.TXT.gz", TMPFILE("PACKAGES.TXT.gz"));
}

void CRefresher::parseInstalled()
{	// Parse the list of installed packages.
	set_title("Parsing");
	_Progress.set_text("Parsing Installed...");

	std::ifstream	_logfile;
	std::string	_inputstr;
	system(Glib::ustring("ls /var/log/packages > " + TMPFILE("installedpackages.log")).c_str());
	_logfile.open(TMPFILE("installedpackages.log").c_str(), std::ios_base::in);
	while ( !_logfile.eof() ) {
		getline(_logfile, _inputstr);
		if ( _inputstr.length() > 0 ) parseFile("/var/log/packages/" + _inputstr);
	}
	_logfile.close();
}

void CRefresher::on_button_cancel()
{	// Every time I try to push her buttons I hit Cancel.
	if ( _Downloader ) delete _Downloader;
	_Downloader = NULL;
	WRITELOG("Mirror download terminated by user");
	response(-4);
}

void CRefresher::parseMirror()
{	// Parse the list of packages provided by a mirror.
	set_title("Parsing");
	_Progress.set_text("Parsing Mirror...");
	parseFile(TMPFILE("PACKAGES.TXT"));
}

bool CRefresher::downloadMirror()
{	// Download the mirror package list, and extract it.
	if ( !_Downloader ) return true;
	float	_status = _Downloader->Iteration();
	if ( _status == -1 && !_DownloadFallback ) {
		delete _Downloader;	// Second download attempt, sans the gz.
		_Downloader = NULL;
		WRITELOG("Failed to download compressed package list, trying uncompressed...");
		_Downloader = new CDownloader(_Mirror + "/PACKAGES.TXT", TMPFILE("PACKAGES.TXT"));
		_DownloadFallback = true;
	} else if ( _status == -1 ) {	// Download is t3h FAIL
		WRITELOG("Error downloading package list from mirror!");
		response(-4);
		delete _Downloader;
		_Downloader = NULL;
		return false;	// Stop calling this function.
	} else if ( _status == 2.0f ) {	// Download finished.
		_Progress.set_fraction(1.0f);
		while ( Gtk::Main::events_pending() ) Gtk::Main::iteration();
		if ( !_DownloadFallback ) system(Glib::ustring("gzip -df " + TMPFILE("PACKAGES.TXT.gz")).c_str());
		delete _Downloader;
		_Downloader = NULL;
		response(1);
		return false;
	} else if ( _status > 0.0f ) {
	       	_Progress.set_text("Downloading Mirror Package List...");
		_Progress.set_fraction(_status);
	}
	return true;
}

bool CRefresher::on_timer()
{	// This allows the GUI to load and enter run() before we actually parse.
	if ( _Parser ) {
		parseInstalled();
		parseMirror();
		response(1);
	} else if ( !_Mirror.find("local://") || !_Mirror.length() )
		response(1);
	else 	return downloadMirror();
	return false;	// Don't repeat the timer.
}

void CRefresher::parseFile(Glib::ustring filename)
{	// Parse a file containing package information.
	std::ifstream	_listfile;
	std::string	_inputstr;
	Glib::ustring	_curpkg;
	static gint32	_pulser = 0;
	bool		_fillDesc;

	_listfile.open(filename.c_str(), std::ios_base::in);
	while ( !_listfile.eof() && _listfile.is_open() ) {
		getline(_listfile, _inputstr);
		if ( !_inputstr.find("PACKAGE NAME:") ) {
			if ( !(++_pulser % 100) ) {	// GUI update.
				_Progress.pulse();
				while ( Gtk::Main::events_pending() ) Gtk::Main::iteration();
			}
			_curpkg = GetPackageName(LTrim(_inputstr.substr(13)));
			if ( !_curpkg.length() ) continue;	// Skip ahead if this is blank.
			(*_Packages)[_curpkg].name = _curpkg;
			if ( !(*_Packages)[_curpkg].description.size() ) _fillDesc = true;
			else _fillDesc = false;
			if ( (gint32)_Blacklist.find("," + _curpkg.lowercase() + ",") >= 0 )
				(*_Packages)[_curpkg].color = COLOR_BLACKLIST;
			else (*_Packages)[_curpkg].color = COLOR_NEUTRAL;
			if ( !filename.find("/var/log") ) {	// installed, else mirror
				(*_Packages)[_curpkg].installedversion = GetPackageVersion(_inputstr.substr(14) + ".tgz");
				(*_Packages)[_curpkg].location = "All";
			} else {
				(*_Packages)[_curpkg].mirrorversion = GetPackageVersion(LTrim(RTrim(_inputstr.substr(14))));
				(*_Packages)[_curpkg].type = _inputstr.substr(_inputstr.rfind('.'));
			}
		} else if ( !_inputstr.find("PACKAGE LOCATION:") && !filename.find(TMPFILE("")) ) {
			(*_Packages)[_curpkg].location = LTrim(_inputstr.substr(17));	// mirror packages only.
			if ( !(*_Packages)[_curpkg].location.find("./") )
				(*_Packages)[_curpkg].location.erase(0, 2);
		} else if ( (!_inputstr.find("PACKAGE SIZE (compressed):") || !_inputstr.find("COMPRESSED PACKAGE SIZE:")) )
			(*_Packages)[_curpkg].packedsize = (guint32)Glib::Ascii::strtod(LTrim(_inputstr.substr(_inputstr.find(':') + 1)));
		else if ( (!_inputstr.find("PACKAGE SIZE (uncompressed):") || !_inputstr.find("UNCOMPRESSED PACKAGE SIZE:")) )
			(*_Packages)[_curpkg].unpackedsize = (guint32)Glib::Ascii::strtod(LTrim(_inputstr.substr(_inputstr.find(':') + 1)));
		else if ( !_inputstr.find("PACKAGE REQUIRED:") ) {
			_inputstr += ",";	
			while ( (int)_inputstr.find(" | ") >= 0 )	// Take care of funky deps.
				_inputstr.replace(_inputstr.find(" | "), 3, ",");
			while ( (int)_inputstr.find(" >") >= 0 )
				_inputstr.erase(_inputstr.find(" >"), _inputstr.find(',', _inputstr.find(" >")) - _inputstr.find(" >"));
			(*_Packages)[_curpkg].deps = LTrim(_inputstr.substr(17));
		} else if ( !_inputstr.find("PACKAGE CONFLICTS:") )
			(*_Packages)[_curpkg].cons = LTrim(_inputstr.substr(_inputstr.find(":") + 1));
		else if ( !_inputstr.find(_curpkg + ":") && (_inputstr.find(':') + 1) < RTrim(_inputstr).length() && _fillDesc )
			(*_Packages)[_curpkg].description.push_back(LTrim(_inputstr.substr(_inputstr.find(':') + 1)));	// Description
	}
	if ( !_listfile.is_open() && !_Mirror.find("local://") && (int)filename.find("PACKAGES.TXT") >= 0 )
		parseLocal();	// This must be a local mirror sans PACKAGES.TXT.

	_listfile.close();
}

void CRefresher::parseLocal()
{	// Searching through a local:// directory recursively for packages and
	// creates the package list that way.  This function should only
	// be called if there is no PACKAGES.TXT in the mirror directory.
	if ( _Mirror.length() < 9 ) return;
	Glib::ustring	_cmd = "find '" + _Mirror.substr(8);	// subtract the local:// part.
	_cmd += "' -regex \".*\\.t[glbx]z\" > " + TMPFILE("localmirrorsearch.tmp");
	system(_cmd.c_str());

	std::ifstream	_ifile;
	std::string	_input;
	_ifile.open(TMPFILE("localmirrorsearch.tmp").c_str(), std::ios_base::in);
	while ( !_ifile.eof() && _ifile.is_open() ) {
		getline(_ifile, _input);
		if ( !_input.length() ) break;
		Glib::ustring	_pkg = GetPackageName(_input.substr(_input.rfind('/') + 1));

		(*_Packages)[_pkg].name = _pkg;
		(*_Packages)[_pkg].mirrorversion = GetPackageVersion(_input.substr(_input.rfind('/') + 1));
		(*_Packages)[_pkg].location = _input.substr(_Mirror.substr(7).length());
		if ( (int)(*_Packages)[_pkg].location.rfind('/') >= 0 )
			(*_Packages)[_pkg].location.erase((*_Packages)[_pkg].location.rfind('/'));
		else	(*_Packages)[_pkg].location = "";
		(*_Packages)[_pkg].location = "local://" + (*_Packages)[_pkg].location;
		(*_Packages)[_pkg].packedsize = 0;
		(*_Packages)[_pkg].unpackedsize = 0;
		if ( (gint32)_Blacklist.find("," + _pkg.lowercase() + ",") >= 0 )
			(*_Packages)[_pkg].color = COLOR_BLACKLIST;
		else (*_Packages)[_pkg].color = COLOR_NEUTRAL;
	}
	system(Glib::ustring("rm -f " + TMPFILE("localmirrorsearch.tmp")).c_str());
}

#endif
